#ifndef _MCFEPORT_H_
#define _MCFEPORT_H_

#ifdef CONFIG_M528x
#include <asm/m528xsim.h>
#endif

/**
 * Currently defines are based on the mcf5282 processor, haven't checked out
 * other processors compatiblity yet- NZG 
 */

enum EPORTTRIGGER{
	LEVEL_TRIG=0,
	RISING_TRIG,
	FALLING_TRIG,
	EITHER_TRIG
	};

enum EPORT_IRQ{
	MCFINT_EPORT1=(MCFINT_VECBASE+1),
	MCFINT_EPORT2,
	MCFINT_EPORT3,
	MCFINT_EPORT4,
	MCFINT_EPORT5,
	MCFINT_EPORT6,
	MCFINT_EPORT7,
};

/**
 * Edge port registers
 */
#define EPPAR (*((volatile u16 *) (MCF_IPSBAR + 0x130000)))
#define EPDDR (*((volatile u8 *) (MCF_IPSBAR + 0x130002)))
#define EPIER (*((volatile u8 *) (MCF_IPSBAR + 0x130003)))
#define EPDR (*((volatile u8 *) (MCF_IPSBAR + 0x130004)))
#define EPPDR (*((volatile u8 *) (MCF_IPSBAR + 0x130005)))
#define EPFR (*((volatile u8 *) (MCF_IPSBAR + 0x130006)))


/**
 * IRQ control masking register
 * Not technically part of the Edge port, but required to enable it's interrupts.
 */
#define IRQMRL (*((volatile unsigned long *) (MCF_IPSBAR+MCFICM_INTC0+MCFINTC_IMRL)))


/**
 * IRQ control functions
 * For an irq to be enabled it must have a trigger assigned, be enabled, and be 
 * umasked.
 * as in:
 *  MCFEPORT_SETUPIRQ
 */


inline void MCFEPORT_SET_TRIGGER(int irq, int trigger){
int shift = irq*2;
EPPAR = (EPPAR&((~3)<<shift))|(trigger<<shift);	
}

inline void MCFEPORT_IRQ_ENABLE(int irq){
EPIER|=(1<<irq);	
}

inline void MCFEPORT_IRQ_DISABLE(int irq){
EPIER&=~(1<<irq);	
}

inline void  MCFEPORT_CLEAR(int irq){
EPFR|=(1<<irq);	
}

inline void MCFEPORT_MASK(int irq){
IRQMRL|=(1<<irq);
}

inline void MCFEPORT_UNMASK(int irq){
IRQMRL&=~(1<<irq);
}

inline int MCFEPORT_PINSTATE(int irq){
return ((EPPDR&(1<<irq))?1:0);
}

inline void MCFEPORT_SETUPIRQ(int irq, int trigger){
	 MCFEPORT_SET_TRIGGER(irq,  trigger);
	 MCFEPORT_IRQ_ENABLE(irq);
	 MCFEPORT_UNMASK(irq);
}

#endif //_MCFEPORT_H_
