/**************************************************************************
*                                                                                         
*  counter.c - generic user space application for manipulating counters                                  
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by      NZG ngustavson@emacinc.com                                 
*                                                                                         
**************************************************************************/
#include <asm/types.h>
#include <linux/counter.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#define DEV_DEFAULT	"/dev/counter"

static void help(void){
const char *help_table = 
"******************************************************\n"
"counter					                               								\n"                        						 	
"EMAC.Inc->NZG                                   							\n"
""__DATE__"                                 										\n"
"usage:                                    										 	\n"   
"counter [-d device -g -h -s -r -w data]  							\n"
"d: specifiy the device node, default is "DEV_DEFAULT" 	\n"
"h: display this help screen											        \n"
"g: enable the counter											           	\n"
"s: disable the counter									                  	\n"
"r: read from the counter													\n"
"w: write to the counter										   				\n"
"ex:																						\n"
"counter -d /dev/count0 -r  												\n"
"reads the current state of /dev/count0:							\n"
"Note: operations are performed in arguement order      \n"
"Therefore operations performed before -d will use			\n"
DEV_DEFAULT"																	\n"
"******************************************************\n";
printf("%s",help_table);
}

inline int OPENDEVICE(char *device, int blocking){
	int fd = open(device?device:DEV_DEFAULT,
		O_RDWR|(blocking?0:O_NONBLOCK));
		return fd;
}


int main(int argc, char *argv[]){
	const char *default_device = DEV_DEFAULT;
	char *device=(char *)default_device;
	int blocking=0;
	int c;
	int operations=0;
	__u16 data;
	
	while ((c = getopt(argc, argv, "d:ghsrw:")) != EOF){
		switch(c){
			case 'h':
				help();
			break;
			case 'd' :
				device = optarg;
			break;
			case 'g':
			{
				int fd = OPENDEVICE(device,0);
				ioctl(fd,ENABLE_COUNT);				
				close(fd);
			}	
			break;	
			case's':
			{
				int fd = OPENDEVICE(device,0);
				ioctl(fd,DISABLE_COUNT);				
				close(fd);
			}	
			break;
			case'r':
			{
				int fd = OPENDEVICE(device,0);
				__u16 data;
				ioctl(fd,READ_COUNT,&data);
				printf("0x%x\n",data);				
				close(fd);
			}	
			break;
			case'w':
				{
				int fd = OPENDEVICE(device,0);
				__u16 data = strtoul(optarg,NULL,0);
				ioctl(fd,WRITE_COUNT,&data);				
				close(fd);
			}	
			break;	
		}
	}	
return 0;
}
