/**************************************************************************
*                                                                                         
*  egp.c - Program for reading/writing/configuring GPIO100 ports
* (by default 4 are contained in the SoM-100ES MAX II CPLD)                                         
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                 
*                                                                         																		 
*   This program is free software; you can redistribute it and/or modify 				 
*   it under the terms of the GNU General Public License as published by  			 
*   the Free Software Foundation; either version 2 of the License, or    					 
*   (at your option) any later version.                                  												 
*                                                                        																			 
**************************************************************************/                                                                                 
#include <stddef.h>
#include <strings.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/egpio.h>
#include <asm/types.h>



#define ARGCHECK(num){if(argc<num){printf("invalid number of arguements\n");help();return -1;}}
#define OPENDEV(filed,dev){if((filed = open(dev, O_RDWR))<=0){printf("couldn't open %s\n",dev);return -1;}}


static int help(void)
{
const char *help_table = 
"**********************************************\n"
"egp					                               								 	\n"                        						 	
"EMAC.Inc->NZG                                   							\n"
""__DATE__"                                 									\n"
"usage:                                    										  	\n"   
"egp device d/c (writearg)												\n"
"commands                                   									 \n"
"                                           												 \n"
"configure      c  (mask)						        	  				 \n"
"configures the pins for input/output or returns     \n"
"the current setting if no mask is specified				\n"
"mask is input/output bitmask 			 						\n"
"1 = output 0 = input                       								 \n"
"ex:																						\n"
"egp /dev/egpio0  c 0xaa												\n"
"sets every other pin to be an output						\n"
"egp /dev/egpio0  c 														\n"
"reads the current pin configuration state				\n"
"                                           												 \n"
"data      d (data)			 				    	       						 \n"
"sets the ouput latch high/low for the according     \n"
"to the data bitmask.														\n"
"If the data bitmask is not specified, the current    \n"
"pin state is retrieved.													\n"
"ex:																						\n"
"egp /dev/egpio1  d 0xff												\n"
"writes the ouput latch all high									\n"
"egp /dev/egpio1  d 														\n"
"reads the current pin state											\n"
"**********************************************\n";
printf("%s",help_table);
return 0;
}

int main(int argc,char **argv)
{
int fd;
__u8 arg[1];
int command;

ARGCHECK(3);

OPENDEV(fd,argv[1]);

switch(*argv[2])//determine the correct ioctl command
{
  case 'c':
  command = (argc<4 )? CONFIGREAD : CONFIGWRITE;
  break;
  case 'd':
  command = (argc<4 )? DATAREAD : DATAWRITE;
  break;
  
  default:
  printf("invalid command - %c\n",argv[2]);
  help();
  return -1;
}
  
  if(argc>3)//get data/mask if available
  	*arg = (__u8)strtoul(argv[3],NULL,0);
  
  if(ioctl(fd,command,arg)<0)//perform the operation
  		printf("ioctl error\n");
  
   if(argc<4)
   		printf("0x%x\n",*arg);//print return value for reads
  
return 0;

}


