/**************************************************************************
*                                                                                         
*  getkey.c  program for mapping keypad input 
* 	to a character matrix.                                          
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                 
*                                                                         																		 
*   This program is free software; you can redistribute it and/or modify 				 
*   it under the terms of the GNU General Public License as published by  			 
*   the Free Software Foundation; either version 2 of the License, or    					 
*   (at your option) any later version.                                  												 
*                                                                        																			 
**************************************************************************/                                                                                 
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <linux/autoconf.h>
#include <linux/keypad.h>

#define DEV_DEFAULT	"/dev/keypad"

static void help(void){
const char *help_table = 
"******************************************************\n"
"getkey					                               								\n"                        						 	
"EMAC.Inc->NZG                                   							\n"
""__DATE__"                                 										\n"
"usage:                                    										 	\n"   
"getkey [-d device -b -g -s file]  								 			\n"
"d: specifiy the device node, default is "DEV_DEFAULT" 	\n"
"b: read blocking, sleep until a key is pressed                  	\n"
"g: get matrix returns the current keypad matrix           	\n"
"s: sets the keypad matrix to the matrix specified in file	\n"
"If a matrix operation is not specified the last key				\n"
"pressed is returned															\n"
"Note:																					\n"
"arguements are evaluated in order 									\n"
"If blocking or device specifications are used, they must	\n"
"be declared before the matrix arguements on the 			\n"
"command line																	\n"
"******************************************************\n";
printf("%s",help_table);
}

static int print_matrix(char *matrix){
	int row,column;
	for(row=0;row<KEYPAD_ROWS;row++){
			for(column=0;column<KEYPAD_COLUMNS;column++)
				printf("%c ",matrix[row*KEYPAD_COLUMNS+column]);
			printf("\n");
	}
	return 0;
}			


static int get_matrix(char *filename,char *matrix){
	char *line=NULL;
	char *element;
	ssize_t len=0;
	int row,column;
	FILE *file;
	
	if((file = fopen(filename,"r"))<=0){
	    printf("couldn't open %s\r\n",filename);
	    return -1;
	  }
	
		for(row=0;row<KEYPAD_ROWS;row++){
			
			if(getline(&line,&len,file)<=0)
				return 0;//no more lines available
			
				
			for(column=0;column<KEYPAD_COLUMNS;column++){
				element = strsep(&line," ");
				if(element==NULL)break;							
				matrix[row*KEYPAD_COLUMNS+column] = *element;
			}		
			
		free(line);
		len=0;
		}
		
		fclose(file);
		
		return 0;
}



inline int OPENDEVICE(char *device, int blocking){
	int fd = open(device?device:DEV_DEFAULT,
		O_RDONLY|(blocking?0:O_NONBLOCK));
		return fd;
}

inline int CHECKFD(int fd, char *device) {
	if(fd<0){ 
	printf("couldn't open %s\n",device?device:DEV_DEFAULT); 
	help(); 
	return -1; 
				}
	return 0;
	}

int main(int argc, char *argv[]){
	char *device=NULL;
	int blocking=0;
	int c;
	
while ((c = getopt(argc, argv, "hd:bgs:")) != EOF){
	switch(c){
		case 'h':
		help();
		return 0;
		case 'd' :
			device = optarg;
			break;
		case 'b':
			blocking=1;
			break;
		case'g':
			{
				int fd = OPENDEVICE(device,blocking);
				char matrix[KEYPAD_ROWS*KEYPAD_COLUMNS];
				bzero(matrix,sizeof(matrix));
				if(CHECKFD(fd,device)<0)
					return 0;
				ioctl(fd,GETKEYARRAY,(unsigned long)matrix);
				print_matrix(matrix);							
				close(fd);
				return 0;
			}	
		case's':
			{
				int fd = OPENDEVICE(device,blocking);
				char matrix[KEYPAD_ROWS*KEYPAD_COLUMNS];
				char *filename = optarg;
				bzero(matrix,sizeof(matrix));
				if(CHECKFD(fd,device)<0)
					return 0;
				if(get_matrix(filename,matrix)<0){
						printf("new matrix not defined\n");
					}
				ioctl(fd,SETKEYARRAY,(unsigned long)matrix);							
				close(fd);
				return 0;
			}
		}	
	}

{//if no matrix operations are done, we arrive here
				int fd = OPENDEVICE(device,blocking);
				char key;
				
				if(CHECKFD(fd,device)<0)
					return 0;
			
				read(fd,&key,1);
				printf("%c\n",key);
				close(fd);
			}
			return 0;


return 0;
}

