/***************************************************************************
                          modreg.c  -  description
                             -------------------
    begin                : Thur Sep 1 2005
    copyright          : (C) 2005 by EMAC.Inc
    email                : ngustavson@emacinc.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <linux/types.h>
#include <modbus.h>
#include <modbus_funcs.h>


#define ARGCHECK(num){if(argc<num){printf("invalid number of arguements\n");help();return -1;}}

#define OPENDEV(filed,dev){if((filed = open(dev, O_RDWR))<=0){printf("couldn't open %s\n",dev);return -1;}}

#define CLEARSTR "clear"

static int help(void)
{
const char *help_table = 
"*************************************************************\n"
"*modreg		\n"
"*EMAC.Inc		\n" 
"*"__DATE__		"\n"   
"*usage:		\n"   
"*modreg port slave address (data)\n"
"*read ex:						\n"
"*modreg /dev/ttyS1 1 5			\n"
"*reads /dev/ttyS1, input register 5 on board 1	\n"
"*write ex:			\n"
"*modreg /dev/ttyS1 1 5 0xaa	\n"
"*writes 0xaa to holding register 5 on board 1 over /dev/ttyS1\n"
"*to read a holding register use -1 for a data value\n"
"*modreg /dev/ttyS1 1 5 -1	\n"
"*************************************************************\n";
printf("%s",help_table);
return 0;
}

static inline void printerror(int err){
	switch(err){
	  case -1: printf("error : unknow function\n");break;
      case -2: printf("crc error\n");break;
      case -3: printf("timeout error\n");break;
      case -4: printf("error : bad slave answer\n");break;
	}
}

int main(int argc, char *argv[]){
  Mbm_trame trame;	
  int MOSI,MISO;
  
   //global control settings
   Mb_verbose=0;
   Mb_485=1;
   Mb_echo=1;
  
  ARGCHECK(4);
  	
 	//trame.device=Mb_open_device(argv[1],19200,1,8,1);
 	
  	//trame.device=Mb_open_device(argv[1],9600,1,8,2);	
  	trame.device=Mb_open_device(argv[1],9600,1,8,2);
  	//trame.device=Mb_open_device(argv[1],19200,1,8,2);	
  	trame.slave = strtoul(argv[2],NULL,0);
  	trame.address = strtoul(argv[3],NULL,0);
	trame.length= 1;
	trame.timeout=500;
 
 	if(argc==4){
 		trame.function=READ_INPUT_REGISTERS;
 		MOSI=0;
 	}
 	else{
 		if((MOSI=strtol(argv[4],NULL,0))==-1)
 			trame.function=READ_HOLDING_REGISTERS;
 		else	
 			trame.function=WRITE_SINGLE_REGISTER;
 		MOSI = strtol(argv[4],NULL,0);
 	}
 
 	{
 		int result=Mb_master(trame,&MOSI,&MISO,NULL,NULL);
 		if(result<0){
 			printerror(result);
 			Mb_close_device(trame.device);
 			return result;
 		}
 	}
 
 	if(trame.function!=WRITE_SINGLE_REGISTER){
 		printf("%x\n",MISO);
 	}
 
  	Mb_close_device(trame.device);
  	return 0;
}

