/**************************************************************************
*                                                                                         
*  overflow.c - user space application demonstrating asychronous notification of
*  a counter overflow.                                
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                 
*                                                                                         
**************************************************************************/
#include <asm/types.h>
#include <linux/counter.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>

#define DEV_DEFAULT	"/dev/counter"

static void help(void){
const char *help_table = 
"******************************************************\n"
"overflow					                               								\n"                        						 	
"EMAC.Inc->NZG                                   							\n"
""__DATE__"                                 										\n"
"usage:                                    										 	\n"   
"overflow [-d device -c countdown]				  					\n"
"d: specifiy the device node, default is "DEV_DEFAULT" 	\n"
"h: display this help screen													\n"
"c: specify a countdown, this will preload the counter		\n"
"so that \"countdown\" bytes occur before overflow			\n"
"and automatically start it													\n"
"																							\n"
"ex:																						\n"
"overflow -d /dev/count0		  												\n"
"notify when an overflow occurs on /dev/count0				\n"
"overflow -d /dev/count1 -c 0x10											\n"
"load the counter with (0 - 0x10) and start it						\n"
"******************************************************\n";
printf("%s",help_table);
}

inline int OPENDEVICE(char *device, int blocking){
	int fd = open(device?device:DEV_DEFAULT,
		O_RDWR|(blocking?0:O_NONBLOCK));
		return fd;
}

volatile int waiting;
const char *default_device = DEV_DEFAULT;
char *device;
int c;
	int fd;
	int oflags;
	struct sigaction overflow_signal;		

static void overflow_handler(int signal){
	printf("\noverflow detected on %s\n",device);
	waiting = 0;
}

int main(int argc, char *argv[]){
	__u16 countdown=0;
	device=(char *)default_device;
	
	while ((c = getopt(argc, argv, "d:hc:")) != EOF){
		switch(c){
			case 'h':
				help();
			return 0;
			case 'd' :
				device = optarg;
			break;
			case 'c':
				countdown = strtoul(optarg,NULL,0);
			break;		
		}
	}	
	
{	
	fd = OPENDEVICE(device,0);
	overflow_signal.sa_handler = overflow_handler;//define signal handler
	sigaction(SIGIO, &overflow_signal,NULL); //attach signal structure to SIGIO
	fcntl(fd, F_SETOWN,getpid()); //set file descriptor ownership to the current process
	oflags = fcntl(fd, F_GETFL); //get current flags from the fd
	fcntl(fd,F_SETFL, oflags|FASYNC);//add FASYNC flag to them

	if(countdown){
		ioctl(fd,DISABLE_COUNT);	
		countdown = 0 - countdown;
		ioctl(fd,WRITE_COUNT,&countdown);
		ioctl(fd,ENABLE_COUNT);	
	}

	waiting=1;
	while(waiting);
	close(fd);
}

return 0;
}



