/**************************************************************************
*                                                                                         
*  setstate.c - Program for setting DtoA state using the WRITEALL ioctl call
*  							Currently only tested for the TLV5614 character driver.                           
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                 
*                                                                         																		 
*   This program is free software; you can redistribute it and/or modify 				 
*   it under the terms of the GNU General Public License as published by  			 
*   the Free Software Foundation; either version 2 of the License, or    					 
*   (at your option) any later version.                                  												 
*                                                                        																			 
**************************************************************************/                                                                                 
#include <stddef.h>
#include <strings.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <asm/types.h>

#define  CHARACTER_INTERFACE
#include <asm/tlv5614.h>

#define BUFFERSIZE 8

#define ARGCHECK(num){if(argc<num){printf("invalid number of arguements\n");help();return -1;}}
#define OPENDEV(filed,dev){if((filed = open(dev, O_RDWR))<=0){printf("couldn't open %s\n",dev);return -1;}}


static int help(void)
{
const char *help_table = 
"*************************************************\n"
"setstate			\n"                        						 	
"EMAC.Inc->NZG    \n"
""__DATE__"       \n"
"usage:           \n"   
"setstate device (state)\n"
"ex:				\n"
"setstate /dev/tlv5614  0x1ff 0x80 -1 0x100 \n"
"writes to DtoA channel A, B, and D.		\n"
"B is not written to because the value is out of\n"
"range										\n"
"setstate /dev/tlv5614		\n"
"returns the current state	\n"
"*************************************************\n";
printf("%s",help_table);
return 0;
}

static inline print_channels(int channels, __u16 *buffer){
	int channel;
	for(channel=0;channel<channels;channel++){
		printf("%x ",buffer[channel]);
	}
}

static void read_command(int fd){
__u16 buffer[NUMCHANNELS];
//printf("read_command\n");
if(ioctl(fd,READALL,&buffer)<0){//perform the operation
  		printf("READALL ioctl error\n");
		return;
  }
print_channels(NUMCHANNELS, buffer);
}


static inline __u16 *argstoarray(__u16 *to,char **from,int n){
int channel;

if(n<NUMCHANNELS){
	printf("invalid number of channels\n");
	help();
	return NULL;
}

for(channel=0;channel<NUMCHANNELS;channel++)
	to[channel] = strtoul(from[channel],NULL,0);
	
return to;	
}

static void write_command(int fd, char **from, int n){
	__u16 buffer[NUMCHANNELS];
	//printf("write command\n");
	argstoarray(buffer,from,n);
	
	if(ioctl(fd,WRITEALL,&buffer)<0)
  		printf("WRITEALL ioctl error\n");;
	
}

int main(int argc,char **argv){
int fd;
__u16 buffer[BUFFERSIZE];
int channels;

//printf("setstate\n");
ARGCHECK(2);
//printf("argcheck passed\n");
OPENDEV(fd,argv[1]);
//printf("device opened\n");

if(argc>2)
	write_command(fd,&argv[2],argc-2);
else
	read_command(fd);
	
close(fd);

return 0;
}	




