#!/bin/sh

# Minix script for testing the SoM-100ES
# Also serves as a demonstration of Minix scripting 
# on the SoM-5282EM
# 07/26/05, NZG

	while test SELECT != "x" ; do
	clear;
	echo "SoM-100ES Test Program V1.0"
	echo "07/26/05 EMAC.Inc"
	echo "depends on EMAC build 1.0 or greater"
	echo "---------------------------------------------"
	echo "K)	Test Keypad";
	echo "L)	Test LCD";
	echo "N)	Test networking";
	echo "P)	Test Digital Ports";
	echo "Q)	Loopback QSPI";
	echo "T)	Read Time from the RTC"
	echo "S)	Set RTC"
	echo "W)	Write all temp storage (/tmp) to MMC/SD card";
	echo "R)	Read all data from MMC/SD card to /tmp";
	echo "---------------------------------------------";
	echo "X)     exit";

	echo -n "Choose a Test ---> ";
	read REPLY;
	SELECT=$REPLY;

	echo "";
	case $SELECT in
		K | k )
			echo "Keypad Test";
			echo "Waiting for Keypress"
			KEY=`getkey -b`
			echo "you pressed" $KEY
			echo "press any key to continue"
			read BLANK;
		;;
		L | l )
			echo "Test LCD";
			echo "Sending \"hello world\" to the LCD"
			echo "hello world">/dev/lcd
			echo "press any key to continue"
			read BLANK;			
		;;
		N | n )
			echo "Pinging EMAC.Inc";
			ping -c 10 www.emacinc.com
			echo "press any key to continue"
			read BLANK;		 
		;;	
		P | p )
			echo "Digital Port Test";
			egp /dev/egpio1 c 0
			DIP=`egp /dev/egpio1 d`
			echo "DIP switches read" $DIP
			echo "Echoing to the high drive ports"							
			egp /dev/egpio2 c 0xff	
			egp /dev/egpio2 d $DIP
			echo "press any key to continue"
			read BLANK;	
								 
		;;
		Q | q )
			echo "perform SPI loopback test";
			echo "Connect MOSI to MISO"
			echo "Press any key when ready"
			read BLANK;
			./spiloop /dev/qspi7
			echo "press any key to continue"
			read BLANK;				 
		;;	
		T | t )
			echo "reading time from hardware clock"
			hwclock
			echo "press any key to continue"
			read BLANK;    					 
		;;			
		S | s )
			echo "setting hardware clock time"
			echo "enter the date as MMDDhhmmYYYY"
			echo "example: 072617332005"
			echo "to set the date to July 26 5:33 PM 2005"
			read NEWTIME
			hwclock --set --date $NEWTIME
			hwclock --hctosys	
			echo "time set"
			echo "time will be stored as long as the module"
			echo "remains in socket"
			echo "press any key to continue"
			read BLANK;				 
		;;		
		
		W | w )
				
			if(grep /mmc /proc/mounts) then
				umount /mmc
			fi
			
			MOUNTED=0
			
			while [ $MOUNTED != 1 ] ; do
			echo "enter the filesystem type, vfat or ext2"
			read FILESYS
			echo "enter the partition number"
			read PARTITION
			mount -t $FILESYS /dev/mmc$PARTITION /mmc
			
			if(grep /mmc /proc/mounts) then
			echo "device mounted"
			MOUNTED=1
			fi
			
			done
			
			echo "copying data from temp storage to MMC/SD"
			cp -r /tmp/* /mmc/			 	
			echo "remounting MMC read-only"
			umount /mmc
			mount -r -t $FILESYS /dev/mmc$PARTITION /mmc
			echo "press any key to continue"
			read BLANK;					 
		;;
		
		R | r )
			
			if(grep /mmc /proc/mounts) then
				umount /mmc
			fi
			
			MOUNTED=0
			
			while [ $MOUNTED != 1 ] ; do
			echo "enter the filesystem type, vfat or ext2"
			read FILESYS
			echo "enter the partition number"
			read PARTITION
			mount -t $FILESYS /dev/mmc$PARTITION /mmc
			
			if(grep /mmc /proc/mounts) then
			echo "device mounted"
			MOUNTED=1
			fi
			
			done
			

			echo "attempting to copy all data from MMC/SD to /tmp"
			cp -r /mmc/* /tmp/
			echo "finished"
			echo "remounting MMC read-only"
			umount /mmc
			mount -r -t $FILESYS /dev/mmc$PARTITION /mmc
			echo "press any key to continue"
			read BLANK;					 
		;;
		
		X | x )
			exit  					 
		;;								
esac
done

